/*
** pork_status.c - status bar implementation.
** Copyright (C) 2002-2003 Amber Adams <amber@ojnk.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include <libfaim/aim.h>
#include <libfaim/faimconfig.h>

#include <pork_util.h>
#include <pork_list.h>
#include <pork_buddy.h>
#include <pork_set.h>
#include <pork_swindow.h>
#include <pork_imwindow.h>
#include <pork_slist.h>
#include <pork_buddy_list.h>
#include <pork_acct.h>
#include <pork_cstr.h>
#include <pork_misc.h>
#include <pork_input.h>
#include <pork_bind.h>
#include <pork_events.h>
#include <pork_screen.h>
#include <pork_screen_cmd.h>
#include <pork_status.h>
#include <pork_format.h>
#include <pork_color.h>

extern struct screen screen;

// Initialization routines for the status bar

int status_init(void) {
	WINDOW *win;

	win = newwin(STATUS_ROWS, screen.cols, screen.rows - STATUS_ROWS, 0);
	if (win == NULL)
		return (-1);

	set_default_win_opts(win);
	screen.status_bar = win;
	return (0);
}

// Draw the status bar, using the format string OPT_FORMAT_STATUS.  This
// parses the format string every time the status bar is redrawn, which
// could waste some cpu time, except that today's computers are sickeningly fast
// and the status bar isn't redrawn all that much anyway.

void status_draw(struct aim_priv *priv) {
	u_int32_t i;
	int s = 0;
	int r_just = 0;
	int llen;
	int rlen = 0;
	char *format = opt_get_str(OPT_FORMAT_STATUS);
	char tmp[CHAR_BUFLEN];
	char l_side[CHAR_BUFLEN];
	char r_side[CHAR_BUFLEN];
	chtype padchar;
	chtype l_cstr[screen.cols + 1];
	chtype r_cstr[screen.cols + 1];

	for (i = 0 ; format[i] != '\0' && s < CHAR_BUFLEN ; i++) {
		if (format[i] == FORMAT_VARIABLE) {
			if (format[i + 1] == '>' && !r_just) {
				l_side[s] = '\0';
				r_just = s;
				i++;
				continue;
			}

			*tmp = '\0';
			status_parse(priv, format[++i], tmp, CHAR_BUFLEN);

			if (*tmp != '\0') {
				int j = 0;

				while (tmp[j] != '\0' && j < CHAR_BUFLEN && s < CHAR_BUFLEN) {
					if (r_just)
						r_side[s - r_just] = tmp[j];
					else
						l_side[s] = tmp[j];

					j++;
					s++;
				}
			}

		} else {
			if (r_just)
				r_side[s - r_just] = format[i];
			else
				l_side[s] = format[i];

			s++;
		}
	}

	if (r_just)
		r_side[s - r_just] = '\0';
	else
		l_side[s] = '\0';

	llen = plaintext_to_cstr(l_side, l_cstr, screen.cols) - 1;

	if (r_just) {
		rlen = plaintext_to_cstr(r_side, r_cstr, screen.cols - llen);
	}

	padchar = l_cstr[llen];

	for (i = llen ; i < screen.cols - rlen ; i++)
		l_cstr[i] = padchar;

	if (r_just) {
		int j = 0;

		for (; i < screen.cols ; i++)
			l_cstr[i] = r_cstr[j++];
	}

	l_cstr[i] = 0;

	mvwputnstr(screen.status_bar, 0, 0, l_cstr, screen.cols);
	wnoutrefresh(screen.status_bar);
}

void status_parse(struct aim_priv *priv, char statchar, char *buf, size_t len) {
	struct imwindow *imwindow = cur_window();
	char *work_format;

	switch (statchar) {
		// Screen name
		case 'n':
			xstrncpy(buf, priv->username, len);
			break;

		// Window name
		case 'w':
			xstrncpy(buf, imwindow->name, len);
			break;

		// Timestamp
		case 'T':
			work_format = opt_get_str(OPT_FORMAT_STATUS_TIMESTAMP);
			make_timestamp(work_format, buf, len);
			break;

		// Activity
		case 'A':
			work_format = opt_get_str(OPT_FORMAT_STATUS_ACTIVITY);
			make_activity(work_format, buf, len);
			break;

		// Typing
		case 'Y':
			work_format = opt_get_str(OPT_FORMAT_STATUS_TYPING);
			make_typing(work_format, buf, len, imwindow);
			break;

		// Held Messages
		case 'H':
			work_format = opt_get_str(OPT_FORMAT_STATUS_HELD);
			make_held(work_format, buf, len, imwindow);
			break;
	}
}
